"""Adds a Help menu"""

settings = {
    "online_help_base_url": {
        "default": "http://osdoc.cogsci.nl",
        "ide": "http://rapunzel.cogsci.nl"
    },
    "online_help_sitemap": {
        "default": "http://osdoc.cogsci.nl/[version]/sitemap.yml",
        "ide": "http://rapunzel.cogsci.nl/sitemap.yml"
    },
    "online_help_psychopy": {
        "default": True,
        "ide": False
    }
}
modes = ["ide", "default"]
