"""Shows a list of example experiments"""

icon = "help-contents"
label = "Example experiments"
menu = {
    "index": -1,
    "separator_after": False,
    "separator_before": True,
    "submenu": "Tools"
}
tooltip = "Shows a list of example experiments"
