# -*- coding:utf-8 -*-

"""
This file is part of OpenSesame.

OpenSesame is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

OpenSesame is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with OpenSesame.  If not, see <http://www.gnu.org/licenses/>.
"""
from libopensesame.py3compat import *
from libopensesame.sketchpad_elements._arrow import Arrow
from libopensesame.sketchpad_elements._circle import Circle
from libopensesame.sketchpad_elements._ellipse import Ellipse
from libopensesame.sketchpad_elements._fixdot import Fixdot
from libopensesame.sketchpad_elements._gabor import Gabor
from libopensesame.sketchpad_elements._image import Image
from libopensesame.sketchpad_elements._line import Line
from libopensesame.sketchpad_elements._noise import Noise
from libopensesame.sketchpad_elements._textline import Textline
from libopensesame.sketchpad_elements._rect import Rect

# Alias for backwards compatibility
arrow = Arrow
circle = Circle
ellipse = Ellipse
fixdot = Fixdot
gabor = Gabor
image = Image
line = Line
noise = Noise
textline = Textline
rect = Rect
