"""Command palette"""

tooltip = "Trigger menu actions"
label = "Command palette"
shortcut = "Ctrl+Shift+P"
icon = "edit-find"
priority = -2
menu = {
    "index": 3,
    "submenu": "View"
}
modes = ["default", "ide"]
