"""Sends a bug report when an exception occurs"""
 
icon = "applications-accessories"
label = "Bug report"
priority = 999999
settings = {
  "bug_report_url": "http://www.cogsci.nl/bug.php"
}
modes = ["default", "ide"]
