"""Notifies the user if an experiment has been created with a different version
of OpenSesame.
"""

icon = "applications-accessories"
label = "OpenSesame 4 notifications"
priority = -1
settings = {
    "os4n_old_experiment_notification": True
}
