"""Turns a tab into a dockwidget"""

icon = "view-fullscreen"
label = "Tab to dockwidget"
priority = -1
menu = {
    "index": 3,
    "separator_after": False,
    "separator_before": False,
    "submenu": "View"
}
toolbar = {
    "index": 9,
    "separator_after": False,
    "separator_before": False
}
tooltip = "Turn tab into dockwidget"
